@@ -33,9 +33,13 @@ use crate::rpc::maintenance::{
 use crate::rpc::watch::{WatchClient, WatchOptions, WatchStream, Watcher};
 #[cfg(feature = "tls")]
 use crate::TlsOptions;
+use http::uri::Uri;
+use std::str::FromStr;
 use std::sync::Arc;
 use std::time::Duration;
-use tonic::transport::Channel;
+use tokio::sync::mpsc::Sender;
+use tonic::transport::{Channel, Endpoint};
+use tower::discover::Change;
 
 const HTTP_PREFIX: &str = "http://";
 const HTTPS_PREFIX: &str = "https://";
@@ -51,6 +55,8 @@ pub struct Client {
     maintenance: MaintenanceClient,
     cluster: ClusterClient,
     election: ElectionClient,
+    options: Option<ConnectOptions>,
+    tx: Sender<Change<Uri, Endpoint>>,
 }
 
 impl Client {
@@ -62,95 +68,130 @@ impl Client {
         let endpoints = {
             let mut eps = Vec::new();
             for e in endpoints.as_ref() {
-                let e = e.as_ref();
-                let mut channel = if e.starts_with(HTTP_PREFIX) {
-                    #[cfg(feature = "tls")]
-                    if let Some(ref connect_options) = options {
-                        if connect_options.tls.is_some() {
-                            return Err(Error::InvalidArgs(String::from(
-                                "TLS options are only supported with HTTPS URLs",
-                            )));
-                        }
-                    }
-
-                    Channel::builder(e.parse()?)
-                } else if e.starts_with(HTTPS_PREFIX) {
-                    #[cfg(not(feature = "tls"))]
-                    return Err(Error::InvalidArgs(String::from(
-                        "HTTPS URLs are only supported with the feature \"tls\"",
-                    )));
-
-                    #[cfg(feature = "tls")]
-                    {
-                        let tls = if let Some(ref connect_options) = options {
-                            connect_options.tls.clone()
-                        } else {
-                            None
-                        }
-                        .unwrap_or_else(TlsOptions::new);
-
-                        Channel::builder(e.parse()?).tls_config(tls)?
-                    }
-                } else {
-                    #[cfg(feature = "tls")]
-                    {
-                        let tls = if let Some(ref connect_options) = options {
-                            connect_options.tls.clone()
-                        } else {
-                            None
-                        };
-
-                        match tls {
-                            Some(tls) => {
-                                let e = HTTPS_PREFIX.to_owned() + e;
-                                Channel::builder(e.parse()?).tls_config(tls)?
-                            }
-                            None => {
-                                let e = HTTP_PREFIX.to_owned() + e;
-                                Channel::builder(e.parse()?)
-                            }
-                        }
-                    }
-
-                    #[cfg(not(feature = "tls"))]
-                    {
-                        let e = HTTP_PREFIX.to_owned() + e;
-                        Channel::builder(e.parse()?)
-                    }
-                };
-
-                let keep_alive = options.as_ref().and_then(|options| options.keep_alive);
-                if let Some((interval, timeout)) = keep_alive {
-                    channel = channel
-                        .keep_alive_while_idle(true)
-                        .http2_keep_alive_interval(interval)
-                        .keep_alive_timeout(timeout);
-                }
-
-                if let Some(timeout) = options.as_ref().and_then(|options| options.timeout) {
-                    channel = channel.timeout(timeout);
-                }
-
+                let channel = Self::build_endpoint(e.as_ref(), &options)?;
                 eps.push(channel);
             }
             eps
         };
 
-        let channel = match endpoints.len() {
-            0 => return Err(Error::InvalidArgs(String::from("empty endpoints"))),
-            1 => endpoints[0].connect_lazy(),
-            _ => Channel::balance_list(endpoints.into_iter()),
+        if endpoints.is_empty() {
+            return Err(Error::InvalidArgs(String::from("empty endpoints")));
+        }
+
+        // Always use balance strategy even if there is only one endpoint.
+        let (channel, tx) = Channel::balance_channel(64);
+        for endpoint in endpoints {
+            // The rx inside `channel` won't be closed or dropped here
+            let _ = tx
+                .send(Change::Insert(endpoint.uri().clone(), endpoint))
+                .await
+                .unwrap();
+        }
+
+        let mut options = options;
+        let auth_token = Self::auth(channel.clone(), &mut options).await?;
+        Ok(Self::build_client(channel, tx, auth_token, options))
+    }
+
+    fn build_endpoint(url: &str, options: &Option<ConnectOptions>) -> Result<Endpoint> {
+        let mut endpoint = if url.starts_with(HTTP_PREFIX) {
+            #[cfg(feature = "tls")]
+            if let Some(connect_options) = options {
+                if connect_options.tls.is_some() {
+                    return Err(Error::InvalidArgs(String::from(
+                        "TLS options are only supported with HTTPS URLs",
+                    )));
+                }
+            }
+
+            Channel::builder(url.parse()?)
+        } else if url.starts_with(HTTPS_PREFIX) {
+            #[cfg(not(feature = "tls"))]
+            return Err(Error::InvalidArgs(String::from(
+                "HTTPS URLs are only supported with the feature \"tls\"",
+            )));
+
+            #[cfg(feature = "tls")]
+            {
+                let tls = if let Some(connect_options) = options {
+                    connect_options.tls.clone()
+                } else {
+                    None
+                }
+                .unwrap_or_else(TlsOptions::new);
+
+                Channel::builder(url.parse()?).tls_config(tls)?
+            }
+        } else {
+            #[cfg(feature = "tls")]
+            {
+                let tls = if let Some(connect_options) = options {
+                    connect_options.tls.clone()
+                } else {
+                    None
+                };
+
+                match tls {
+                    Some(tls) => {
+                        let e = HTTPS_PREFIX.to_owned() + url;
+                        Channel::builder(e.parse()?).tls_config(tls)?
+                    }
+                    None => {
+                        let e = HTTP_PREFIX.to_owned() + url;
+                        Channel::builder(e.parse()?)
+                    }
+                }
+            }
+
+            #[cfg(not(feature = "tls"))]
+            {
+                let e = HTTP_PREFIX.to_owned() + url;
+                Channel::builder(e.parse()?)
+            }
         };
 
-        let auth_token: Option<Arc<http::HeaderValue>> =
-            if let Some((name, password)) = options.and_then(|options| options.user) {
-                let mut tmp_auth = AuthClient::new(channel.clone(), None);
-                let resp = tmp_auth.authenticate(name, password).await?;
-                Some(Arc::new(resp.token().parse()?))
-            } else {
-                None
-            };
+        let keep_alive = options.as_ref().and_then(|options| options.keep_alive);
+        if let Some((interval, timeout)) = keep_alive {
+            endpoint = endpoint
+                .keep_alive_while_idle(true)
+                .http2_keep_alive_interval(interval)
+                .keep_alive_timeout(timeout);
+        }
 
+        if let Some(timeout) = options.as_ref().and_then(|options| options.timeout) {
+            endpoint = endpoint.timeout(timeout);
+        }
+
+        Ok(endpoint)
+    }
+
+    async fn auth(
+        channel: Channel,
+        options: &mut Option<ConnectOptions>,
+    ) -> Result<Option<Arc<http::HeaderValue>>> {
+        let user = match options {
+            None => return Ok(None),
+            Some(opt) => {
+                // Take away the user, the password should not be stored in client.
+                opt.user.take()
+            }
+        };
+
+        if let Some((name, password)) = user {
+            let mut tmp_auth = AuthClient::new(channel, None);
+            let resp = tmp_auth.authenticate(name, password).await?;
+            Ok(Some(Arc::new(resp.token().parse()?)))
+        } else {
+            Ok(None)
+        }
+    }
+
+    fn build_client(
+        channel: Channel,
+        tx: Sender<Change<Uri, Endpoint>>,
+        auth_token: Option<Arc<http::HeaderValue>>,
+        options: Option<ConnectOptions>,
+    ) -> Self {
         let kv = KvClient::new(channel.clone(), auth_token.clone());
         let watch = WatchClient::new(channel.clone(), auth_token.clone());
         let lease = LeaseClient::new(channel.clone(), auth_token.clone());
@@ -160,7 +201,7 @@ impl Client {
         let maintenance = MaintenanceClient::new(channel.clone(), auth_token.clone());
         let election = ElectionClient::new(channel, auth_token);
 
-        Ok(Self {
+        Self {
             kv,
             watch,
             lease,
@@ -169,6 +210,41 @@ impl Client {
             maintenance,
             cluster,
             election,
+            options,
+            tx,
+        }
+    }
+
+    /// Dynamically add an endpoint to the client.
+    ///
+    /// Which can be used to add a new member to the underlying balance cache.
+    /// The typical scenario is that application can use a services discovery
+    /// to discover the member list changes and add/remove them to/from the client.
+    ///
+    /// Note that the [`Client`] doesn't check the authentication before added.
+    /// So the etcd member of the added endpoint REQUIRES to use the same auth
+    /// token as when create the client. Otherwise, the underlying balance
+    /// services will not be able to connect to the new endpoint.
+    #[inline]
+    pub async fn add_endpoint<E: AsRef<str>>(&self, endpoint: E) -> Result<()> {
+        let endpoint = Self::build_endpoint(endpoint.as_ref(), &self.options)?;
+        let tx = &self.tx;
+        tx.send(Change::Insert(endpoint.uri().clone(), endpoint))
+            .await
+            .map_err(|e| Error::EndpointError(format!("failed to add endpoint because of {}", e)))
+    }
+
+    /// Dynamically remove an endpoint from the client.
+    ///
+    /// Note that the `endpoint` str should be the same as it was added.
+    /// And the underlying balance services cache used the hash from the Uri,
+    /// which was parsed from `endpoint` str, to do the equality comparisons.
+    #[inline]
+    pub async fn remove_endpoint<E: AsRef<str>>(&self, endpoint: E) -> Result<()> {
+        let uri = http::Uri::from_str(endpoint.as_ref())?;
+        let tx = &self.tx;
+        tx.send(Change::Remove(uri)).await.map_err(|e| {
+            Error::EndpointError(format!("failed to remove endpoint because of {}", e))
         })
     }
 
@@ -678,9 +754,11 @@ mod tests {
     use super::*;
     use crate::{Compare, CompareOp, EventType, PermissionType, TxnOp, TxnOpResponse};
 
+    const DEFAULT_TEST_ENDPOINT: &str = "localhost:2379";
+
     /// Get client for testing.
     async fn get_client() -> Result<Client> {
-        Client::connect(["localhost:2379"], None).await
+        Client::connect([DEFAULT_TEST_ENDPOINT], None).await
     }
 
     #[tokio::test]
@@ -1413,4 +1491,37 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_remove_and_add_endpoint() -> Result<()> {
+        let mut client = get_client().await?;
+        client.put("endpoint", "add_remove", None).await?;
+
+        // get key
+        {
+            let resp = client.get("endpoint", None).await?;
+            assert_eq!(resp.count(), 1);
+            assert!(!resp.more());
+            assert_eq!(resp.kvs().len(), 1);
+            assert_eq!(resp.kvs()[0].key(), b"endpoint");
+            assert_eq!(resp.kvs()[0].value(), b"add_remove");
+        }
+
+        // remove endpoint
+        client.remove_endpoint(DEFAULT_TEST_ENDPOINT).await?;
+        // `Client::get` will hang before adding the endpoint back
+        client.add_endpoint(DEFAULT_TEST_ENDPOINT).await?;
+
+        // get key after remove and add endpoint
+        {
+            let resp = client.get("endpoint", None).await?;
+            assert_eq!(resp.count(), 1);
+            assert!(!resp.more());
+            assert_eq!(resp.kvs().len(), 1);
+            assert_eq!(resp.kvs()[0].key(), b"endpoint");
+            assert_eq!(resp.kvs()[0].value(), b"add_remove");
+        }
+
+        Ok(())
+    }
 }
