@@ -3,6 +3,7 @@
 pub use crate::rpc::pb::etcdserverpb::compare::CompareResult as CompareOp;
 pub use crate::rpc::pb::etcdserverpb::range_request::{SortOrder, SortTarget};
 
+use crate::auth::AuthService;
 use crate::error::Result;
 use crate::rpc::pb::etcdserverpb::compare::{CompareTarget, TargetUnion};
 use crate::rpc::pb::etcdserverpb::kv_client::KvClient as PbKvClient;
@@ -17,25 +18,23 @@ use crate::rpc::pb::etcdserverpb::{
     RequestOp as PbTxnRequestOp, TxnRequest as PbTxnRequest, TxnResponse as PbTxnResponse,
 };
 use crate::rpc::{get_prefix, KeyRange, KeyValue, ResponseHeader};
+use http::HeaderValue;
+use std::sync::Arc;
 use tonic::transport::Channel;
-use tonic::{Interceptor, IntoRequest, Request};
+use tonic::{IntoRequest, Request};
 
 /// Client for KV operations.
 #[repr(transparent)]
 #[derive(Clone)]
 pub struct KvClient {
-    inner: PbKvClient<Channel>,
+    inner: PbKvClient<AuthService<Channel>>,
 }
 
 impl KvClient {
     /// Creates a kv client.
     #[inline]
-    pub(crate) fn new(channel: Channel, interceptor: Option<Interceptor>) -> Self {
-        let inner = match interceptor {
-            Some(it) => PbKvClient::with_interceptor(channel, it),
-            None => PbKvClient::new(channel),
-        };
-
+    pub(crate) fn new(channel: Channel, auth_token: Option<Arc<HeaderValue>>) -> Self {
+        let inner = PbKvClient::new(AuthService::new(channel, auth_token));
         Self { inner }
     }
 
@@ -190,6 +189,7 @@ impl IntoRequest<PbPutRequest> for PutOptions {
 }
 
 /// Response for `Put` operation.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 #[repr(transparent)]
 pub struct PutResponse(PbPutResponse);
@@ -406,6 +406,7 @@ impl IntoRequest<PbRangeRequest> for GetOptions {
 }
 
 /// Response for `Get` operation.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 #[repr(transparent)]
 pub struct GetResponse(PbRangeResponse);
@@ -532,6 +533,7 @@ impl IntoRequest<PbDeleteRequest> for DeleteOptions {
 }
 
 /// Response for `Delete` operation.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 #[repr(transparent)]
 pub struct DeleteResponse(PbDeleteResponse);
@@ -608,6 +610,7 @@ impl IntoRequest<PbCompactionRequest> for CompactionOptions {
 }
 
 /// Response for `Compact` operation.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 #[repr(transparent)]
 pub struct CompactionResponse(PbCompactionResponse);
@@ -858,6 +861,7 @@ impl IntoRequest<PbTxnRequest> for Txn {
 }
 
 /// Transaction operation response.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 pub enum TxnOpResponse {
     Put(PutResponse),
@@ -867,6 +871,7 @@ pub enum TxnOpResponse {
 }
 
 /// Response for `Txn` operation.
+#[cfg_attr(feature = "pub-response-field", visible::StructFields(pub))]
 #[derive(Debug, Clone)]
 #[repr(transparent)]
 pub struct TxnResponse(PbTxnResponse);
